<?php

namespace Lean\Console\Form;

use Picea\Ui\Method\{ FormInterface, FormContext, FormContextInterface };

use \Lean\Console\Lib;

use Picea\Compiler\Context;
use Psr\Http\Message\ServerRequestInterface;
use \Ulmus\Entity\InformationSchema\Table;

class Database implements FormInterface
{
    protected ? Lib\DatabaseMigrations $migration;

    public function __construct(Lib\DatabaseMigrations $migration)
    {
        $this->migration = $migration;
    }

    public function initialize(FormContextInterface $context) : void
    {
        $context->tableExist = [];

        foreach($this->migration->entities as $entity => $table) {
            $adapter = $entity::resolveEntity()->sqlAdapter()->adapter();

            $tableName = $table->tableName();
            $databaseName = $table->databaseName();

            if ( ! $tableName || ! $databaseName ) {
                continue;
            }

            $tableEntity = $adapter->schemaTable($databaseName, $tableName);

            if ( $tableEntity ) {
                #$fields = $entiy::resolveEntity()->fieldList();

                foreach($tableEntity->columns as $col) {
                   //dump( $entity::field($col->name) );
                }

                # if columns are different
                $context->status[$entity] = [
                    'msg' => "up-to-date",
                    'query' => "",
                ];
            }
            else {
                $context->status[$entity] = [
                    'msg' => "unexisting",
                    'query' => $entity::repository()->createSqlQuery()->getSqlQuery(true),
                ];

            }
        }

        if ( $context->formSent() ) {

        }
    }

    public function validate(FormContextInterface $context) : bool
    {
        return $context->valid();
    }

    public function execute(FormContextInterface $context) : void
    {
        if ( $context->create ?? false ) {
            $context->create::repository()->createTable();
        }
        elseif ( $context->createAll ?? false ) {
            foreach(explode(',', $context->createAll) as $entity) {
                $entity::repository()->createTable();
            }
        }

        $this->initialize($context);
    }

    public function getContext(ServerRequestInterface $request) : FormContextInterface
    {
        return new class($request) extends FormContext {

            public array $status = [];

            public array $actions = [];
        };
    }
}